/*****************************************************************************
 *   Copyright (C) 2009 Peter "woggle" Mack, mac@denich.net                  *
 *   based on the key handling by Peter Dannegger                            *
 *     see www.mikrocontroller.net                                           *
 *   Copyright (C) 2011 Christian Brandtner brandtner@brandtner.net          *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License.          *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.               *
 *   																		 *
 *   22.03.2011 Zeitgesteuerte Displaybeleuchtung C.B. 						 *
 *                                                                           *
 *****************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <string.h>
#include <inttypes.h>



#include "main.h"

#include "timer.h"
#include "eeprom.h"
#include "lcd.h"
#include "uart1.h"


volatile uint16_t timer;
volatile uint16_t abo_timer;
volatile uint8_t pwm1 = 255;
volatile static unsigned int tim_main;

#define FRAME_LENGTH 20
uint16_t icrval = (F_CPU / 64) * FRAME_LENGTH / 1000 ;
uint16_t minocr_a = (F_CPU / 64) * 1/ 1000;	// 312
uint16_t maxocr_a = (F_CPU / 64) * 2/ 1000;	// 624
uint16_t minocr_b = (F_CPU / 64) * 1/ 1000;	// 312
uint16_t maxocr_b = (F_CPU / 64) * 2/ 1000;	// 624

uint8_t key_state = 0;		// debounced and inverted key state:
							// bit = 1: key pressed
uint8_t key_press = 0;		// key press detect
uint8_t key_rpt;			// key long press and repeat
volatile uint8_t key_repeat_next;
volatile uint8_t key_counter;



uint16_t DisplayTime = 0;	// Leuchtdauer
volatile uint16_t IdleTimer = 0;		// InaktivitätsTimer

uint8_t WarnCount=0;		// Zähler der LIPO Warnzeit
uint16_t WarnToggle=0;		// Togglezähler zum blinken
uint16_t WarnTime=15;		// Länge der LIPO Warnzeit 15 Sek.
uint16_t LipoOK=0;			// Zeit für Lipo Ok Erkennung


volatile uint8_t Display_on;// Flag Display on/off

volatile unsigned int beeptime = 0;
unsigned int BeepMuster = 0xffff;

volatile unsigned int CountMilliseconds = 0;
//volatile static unsigned int tim_main;
volatile unsigned char UpdateMotor = 0;
//volatile unsigned int cntKompass = 0;
// Size of Buffer for Converting unsigned int Value to ASCII

#define STRING_BUFFER_SIZE 5

// Buffer for Converting unsigned int Value to ASCII
char String_Buffer[STRING_BUFFER_SIZE];

/****************************************************************************/



enum {
  STOP             = 0,
  CK               = 1,
  CK8              = 2,
  CK64             = 3,
  CK256            = 4,
  CK1024           = 5,
  T0_FALLING_EDGE  = 6,
  T0_RISING_EDGE   = 7
};



/****************************************************************************
  Function definitions
****************************************************************************/
unsigned char Recognize_Event(void);



//*****************************************************************************
// 


#if defined (__AVR_ATmega32__)
ISR(TIMER0_COMP_vect)						// Timer-Interrupt (100 Hz)
#else
ISR(TIMER0_COMPA_vect)					// Timer-Interrupt (100 Hz)
#endif
{
  static uint8_t ct0 = 0;
  static uint8_t ct1 = 0;
  static uint8_t rpt = 0;
  uint8_t i;
  static unsigned char cnt_1ms = 1,cnt = 0;
  unsigned char pieper_ein = 0;

	// Key handling by Peter Dannegger
	// see www.mikrocontroller.net
	i = key_state ^ ~KEY_PIN;		// key changed ?
	ct0 = ~(ct0 & i);				// reset or count ct0
	ct1 = ct0 ^ (ct1 & i);			// reset or count ct1
	i &= (ct0 & ct1);				// count until roll over ?
	key_state ^= i;					// then toggle debounced state
	key_press |= (key_state & i);	// 0->1: key press detect

	IdleTimer ++;					// nix zu tun? Timer hochzählen

	if (IdleTimer == 12000)			// Warnhinweis
	  {
		beeptime = 200;
	    BeepMuster = 0x0080;
	    IdleTimer = 0;
	  }

	if(!cnt--)
	    {
	     cnt = 9;
	     CountMilliseconds++;
	     cnt_1ms++;
	    }

	if (i!=0)


		{									// Displaylicht einschalten, und bzw. Timeoutzählerreset wenn Taste gedrückt wurde
		if (Display_on ==0)set_D_LIGHT();	// einschalten
		  							// if Display_on ==0
		Display_on =1;				// Flag Display on
		DisplayTime = 0;			// Timer Reset
		IdleTimer = 0;			// Idletimeout Reset

		}

	if (DisplayTimeout > 0)
		{
		 if (Display_on==1)
			{
			 DisplayTime++;
			  if ((DisplayTime/100) == DisplayTimeout)  //ISR läuft mit 100Hz

			   {								//Displaylicht ausschalten
			 	Display_on = 0;				// Flag Display off
				clr_D_LIGHT();
			   }
		    }
		}

#ifdef HWVERSION3_9
	LipoCheck();					// Lipo prüfen
#endif

	if(beeptime)
       {
       if(beeptime > 10) beeptime -= 10; else beeptime = 0;
       if(beeptime & BeepMuster) pieper_ein = 1; else pieper_ein = 0;
       }
    else
     {
      pieper_ein = 0;
      BeepMuster = 0xffff;
     }

    if(pieper_ein) set_BEEP(); else clr_BEEP();


	if ((key_state & REPEAT_MASK) == 0)	// check repeat function
	{
		rpt = REPEAT_START;				// start delay
		key_repeat_next = REPEAT_NEXT;                         // Wert bestimmt Tasten-Beschleunigung/-Wiederholrate beginnt, mit 200ms

	}
	if (--rpt == 0)
	{
		rpt = REPEAT_NEXT;				// repeat delay
		key_rpt |= (key_state & REPEAT_MASK);
	}

	
	if (timer > 0) 	timer --;

	if (abo_timer > 0) abo_timer --;

}

#ifdef HWVERSION3_9

void LipoCheck (void) 	// Lowbatpin des Spannungswandlers prüfen
						// LBO des LT1308 wechselt zum Ende der Batterielaufzeit häufig seinen Zustand in der Übergangsphase zum LowBat
						// Die Akkuspannung schwankt auch abhängig vom momentanen Stromverbrauch
{
//	if (WarnToggle>0)lcd_print_hex_at(0,0,WarnCount,0);
	if (WarnToggle==1)  // Beim ersten Auftreten Warnung ausgeben, Rythmus 5/10 Sekunden
		{
		 beeptime = 1000;
		 BeepMuster = 0x0020;
		 lcd_printpns_at (0, 0, PSTR("  LIPO  !!Warning!!  "),0);					// Warnung wechselnd anzeigen

		}
	if (WarnToggle==500) lcd_printpns_at (0, 0, PSTR("  LIPO  !!Warning!!  "),2);   // Warnung wechselnd anzeigen
    if (WarnToggle == 1000) WarnToggle = 0;		//erstmal bis hier warnen

	if (WarnToggle > 0)WarnToggle++;		// weiter hochzählen

	if ((PINC & (1<<LowBat)))	// Kurzzeitige Unterspannung bearbeiten und Warnung ausgeben
		{
		 if (WarnCount > 0)	  WarnCount--;		// Bei LIPO OK erstmal runterzählen, LT1308 überlegt sich noch genauer ob nun ok oder nicht
		 if (LipoOK > 0) LipoOK++;				//Lipo OK Zähler starten
		}
	if (LipoOK==12000)			// nach Ablauf der letzen Unterspannungswarnung(2 Min.) Info Lipo OK
		{
		lcd_printpns_at (0, 0, PSTR("       LIPO OK       "),2);

		beeptime = 500;
		BeepMuster = 0x00c0;
		LipoOK = 0;				// keine weiteren Hinweise mehr das Lipo ok ist, einmal ist gut
		}
	if (!(PINC & (1<<LowBat)) )		// LT 1308 Unterspannung erkannt
	{
	  WarnCount++;					//solange LBO low ist Zähler hochzählen
	  LipoOK = 1;					// Lipo OK-Zähler reset
      if (WarnCount == 1 && WarnToggle == 0 )WarnToggle =1;  // Warnhinweis starten
    }

	if ((WarnCount/10) == WarnTime) clr_V_On();		// nu ist alle, Spannung abschalten



}

#endif


// -----------------------------------------------------------------------
unsigned int SetDelay (unsigned int t)
{
//  TIMSK0 &= ~_BV(TOIE0);
  return(CountMilliseconds + t + 1);
//  TIMSK0 |= _BV(TOIE0);
}

// -----------------------------------------------------------------------
char CheckDelay(unsigned int t)
{
//  TIMSK0 &= ~_BV(TOIE0);
  return(((t - CountMilliseconds) & 0x8000) >> 9);
//  TIMSK0 |= _BV(TOIE0);
}

// -----------------------------------------------------------------------
void Delay_ms(unsigned int w)
{
 unsigned int akt;
 akt = SetDelay(w);
 while (!CheckDelay(akt));
}


void TIMER1_Init (void) /*Servos*/
{
    DDRD |= (1<<PORTD4)|(1<<PORTD5);
    TCCR1A |= (1<<COM1B1) |(1<<COM1A1) | (1<<WGM11);
    TCCR1B |= (1<<CS11)|(1<<CS10)| (1<<WGM12)| (1<<WGM13);
    
    ICR1 = icrval;
	OCR1A = minocr_a + ((maxocr_a-minocr_a)/2);
	OCR1B = minocr_b + ((maxocr_b-minocr_b)/2);
}


void set_pwm_a(uint16_t value)
{
	uint16_t setv = ((value * 9) / 10 ) + minocr_a;
	
	if((setv > 311)&&(setv < 637))
	{
		OCR1A = setv;
	}
}
void set_pwm_b(uint8_t value)
{
	uint16_t setv = ((value * 4 * 5 ) / 6 ) + minocr_b; 
	if((setv > 200)&&(setv < 750))
	{
		OCR1B = setv;
	}
}
    


//*****************************************************************************
// 
void Timer0_Init (void)
{
	timer = 0;

#if defined (__AVR_ATmega32__)
	TCCR0 = (1 << CS02) | (1 << CS00) | (1 << WGM01);		// Prescaler 1024
	OCR0 = (F_CPU / (100L * 1024L)) ;

	TIMSK |= (1 << OCIE0);	// enable interrupt for OCR
#else
	TCCR0A = (1 << WGM01);
	TCCR0B = (1 << CS02) | (1 << CS00);
	OCR0A = (F_CPU / (100L * 1024L)) ;

	TIMSK0 |= (1 << OCIE0A);	// enable interrupt for OCR
#endif
}


//*****************************************************************************
//
// check if a key has been pressed. Each pressed key is reported
// only once
//

uint8_t get_key_press (uint8_t key_mask)
{
	uint8_t sreg = SREG;

	// disable all interrupts
	cli();

  key_mask &= key_press;	// read key(s)
  key_press ^= key_mask;	// clear key(s)


	SREG = sreg;	// restore status register

  return key_mask;
}



uint8_t get_key_rpt (uint8_t key_mask)
//
// check if a key has been pressed long enough such that the
// key repeat functionality kicks in. After a small setup delay
// the key is reported beeing pressed in subsequent calls
// to this function. This simulates the user repeatedly
// pressing and releasing the key.
//

{
	uint8_t sreg = SREG;

	// disable all interrupts
	cli();
	
  key_mask &= key_rpt;	// read key(s)
  key_rpt ^= key_mask;	// clear key(s)
	
	SREG = sreg;	// restore status register
	
  return key_mask;
}


//*****************************************************************************
// 
uint8_t get_key_short (uint8_t key_mask)
{
	uint8_t ret;
	uint8_t sreg = SREG;
	
	// disable all interrupts
	cli();
	
  ret = get_key_press (~key_state & key_mask);
	
	SREG = sreg;	// restore status register
	
  return ret;
}


//*****************************************************************************
// 
uint8_t get_key_long (uint8_t key_mask)
{
  return get_key_press (get_key_rpt (key_mask));
}


//*****************************************************************************
// 
uint8_t get_key_long2 (uint8_t key_mask)
{
  return get_key_press (get_key_rpt (key_press^key_mask));
}


//*****************************************************************************
// 
uint8_t get_key_long_rpt (uint8_t key_mask)
{
  return get_key_rpt (~key_press^key_mask);
}


